import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _typeof from "@babel/runtime/helpers/typeof";
var _excluded = ["noResultsMessage", "noOptionsMessage", "onChange", "options", "value", "simpleValue", "placeholder", "isSearchable", "isDisabled", "isClearable", "isMulti", "isFetching", "onInputChange", "loadingMessage", "menuPortalTarget", "menuIsPortal", "originalOptions"],
    _excluded2 = ["showMoreLabel", "showLessLabel", "simpleValue", "loadingMessage", "updatingMessage", "options", "menuIsPortal", "placeholder", "isSearchable", "isClearable", "noResultsMessage", "noOptionsMessage"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

import React, { useRef, useState } from 'react';
import PropTypes from 'prop-types';
import DataDrivenSelect, { flatOptions } from '@data-driven-forms/common/select';
import parseInternalValue from '@data-driven-forms/common/select/parse-internal-value';
import Downshift from 'downshift';
import CaretDownIcon from "@patternfly/react-icons/dist/dynamic/icons/caret-down-icon";
import CloseIcon from "@patternfly/react-icons/dist/dynamic/icons/close-icon";
import CircleNotchIcon from "@patternfly/react-icons/dist/dynamic/icons/circle-notch-icon";
import './select-styles.css';
import '@patternfly/react-styles/css/components/Select/select.css';
import '@patternfly/react-styles/css/components/Chip/chip.css';
import '@patternfly/react-styles/css/components/Chip/chip-group.css';
import '@patternfly/react-styles/css/components/Divider/divider.css';
import Menu from './menu';
import ClearIndicator from './clear-indicator';
import ValueContainer from './value-container';
import { Icon } from "@patternfly/react-core/dist/dynamic/components/Icon";

var _itemToString = function itemToString(value, isMulti, showMore, handleShowMore, handleChange) {
  if (!value) {
    return '';
  }

  if (Array.isArray(value)) {
    if (!value || value.length === 0) {
      return;
    }

    if (isMulti) {
      var visibleOptions = showMore ? value : value.slice(0, 3);
      return /*#__PURE__*/React.createElement("div", {
        className: "pf-v5-c-chip-group pf-v5-u-ml-sm",
        onClick: function onClick(event) {
          return event.stopPropagation();
        }
      }, /*#__PURE__*/React.createElement("ul", {
        className: "pf-v5-c-chip-group__list",
        "aria-label": "Chip group category"
      }, visibleOptions.map(function (item, index) {
        var label = _typeof(item) === 'object' ? item.label : item;
        return /*#__PURE__*/React.createElement("li", {
          className: "pf-v5-c-chip-group__list-item",
          onClick: function onClick(event) {
            return event.stopPropagation();
          },
          key: item.key || item.value || item
        }, /*#__PURE__*/React.createElement("div", {
          className: "pf-v5-c-chip"
        }, /*#__PURE__*/React.createElement("span", {
          className: "pf-v5-c-chip__text",
          id: "pf-random-id-".concat(index, "-").concat(label)
        }, label), /*#__PURE__*/React.createElement("button", {
          onClick: function onClick() {
            return handleChange(item);
          },
          className: "pf-v5-c-button pf-u-plain pf-v5-u-pt-0 pf-v5-u-pb-0",
          type: "button",
          "aria-label": "remove option"
        }, /*#__PURE__*/React.createElement(CloseIcon, null))));
      }), value.length > 3 && /*#__PURE__*/React.createElement("li", {
        className: "pf-v5-c-chip-group__list-item"
      }, /*#__PURE__*/React.createElement("button", {
        type: "button",
        onClick: handleShowMore,
        className: "pf-v5-c-chip pf-m-overflow"
      }, /*#__PURE__*/React.createElement("span", {
        className: "pf-v5-c-chip__text"
      }, showMore ? 'Show less' : "".concat(value.length - 3, " more"))))));
    }

    return value.map(function (item) {
      return _typeof(item) === 'object' ? item.label : item;
    });
  }

  if (_typeof(value) === 'object') {
    return value.label;
  }

  return value;
}; // TODO fix the value of internal select not to be an array all the time. It forces the filter value to be an array and it crashes sometimes.


var filterOptions = function filterOptions(options) {
  var filterValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var filter = (Array.isArray(filterValue) && filterValue.length > 0 ? filterValue[0] : filterValue).toLowerCase();

  if (!filter) {
    return options;
  }

  return options.map(function (option) {
    var _option$label2;

    if (option.options) {
      var filteredNested = option.options.map(function (option) {
        var _option$label;

        return (_option$label = option.label) !== null && _option$label !== void 0 && _option$label.toLowerCase().includes(filter) ? option : null;
      }).filter(Boolean);

      if (filteredNested.length === 0) {
        return null;
      }

      return _objectSpread(_objectSpread({}, option), {}, {
        options: filteredNested
      });
    }

    if ((_option$label2 = option.label) !== null && _option$label2 !== void 0 && _option$label2.toLowerCase().includes(filter)) {
      return option;
    }

    return null;
  }).filter(Boolean);
};

var getValue = function getValue(isMulti, option, value) {
  if (!isMulti || !option) {
    return option;
  }

  var isSelected = value.find(function (_ref) {
    var value = _ref.value;
    return value === option.value;
  });
  return isSelected ? value.filter(function (_ref2) {
    var value = _ref2.value;
    return value !== option.value;
  }) : [].concat(_toConsumableArray(value), [option]);
};

var _stateReducer = function stateReducer(state, changes, isMulti) {
  switch (changes.type) {
    case Downshift.stateChangeTypes.clickButton:
      return _objectSpread(_objectSpread(_objectSpread({}, state), changes), {}, {
        highlightedIndex: undefined,
        // reset the item focus to prevent initial scroll and portal menu warping
        inputValue: undefined
      });

    case Downshift.stateChangeTypes.keyDownEnter:
    case Downshift.stateChangeTypes.clickItem:
      return _objectSpread(_objectSpread({}, changes), {}, {
        isOpen: isMulti ? state.isOpen : !state.isOpen,
        highlightedIndex: state.highlightedIndex,
        inputValue: isMulti ? state.inputValue : changes.inputValue // prevent filter value change after option click

      });

    case Downshift.stateChangeTypes.controlledPropUpdatedSelectedItem:
      return _objectSpread(_objectSpread({}, changes), {}, {
        inputValue: state.inputValue
      });

    case Downshift.stateChangeTypes.mouseUp:
      if (typeof changes.inputValue === 'string') {
        return _objectSpread({}, changes);
      }

      if (Array.isArray(changes.inputValue) && typeof changes.inputValue[0] === 'string') {
        return _objectSpread(_objectSpread({}, changes), {}, {
          inputValue: changes.inputValue[0]
        });
      }

      if (state.isOpen === true && changes.isOpen === false && changes.inputValue) {
        return _objectSpread(_objectSpread(_objectSpread({}, state), changes), {}, {
          inputValue: ''
        });
      }

      return _objectSpread(_objectSpread({}, changes), {}, {
        inputValue: state.inputValue
      });

    default:
      return changes;
  }
};

var InternalSelect = function InternalSelect(_ref3) {
  var noResultsMessage = _ref3.noResultsMessage,
      noOptionsMessage = _ref3.noOptionsMessage,
      onChange = _ref3.onChange,
      options = _ref3.options,
      value = _ref3.value,
      simpleValue = _ref3.simpleValue,
      placeholder = _ref3.placeholder,
      isSearchable = _ref3.isSearchable,
      isDisabled = _ref3.isDisabled,
      isClearable = _ref3.isClearable,
      isMulti = _ref3.isMulti,
      isFetching = _ref3.isFetching,
      onInputChange = _ref3.onInputChange,
      loadingMessage = _ref3.loadingMessage,
      menuPortalTarget = _ref3.menuPortalTarget,
      menuIsPortal = _ref3.menuIsPortal,
      originalOptions = _ref3.originalOptions,
      props = _objectWithoutProperties(_ref3, _excluded);

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      showMore = _useState2[0],
      setShowMore = _useState2[1];

  var inputRef = useRef();
  var selectToggleRef = useRef();
  var parsedValue = parseInternalValue(value);

  var handleShowMore = function handleShowMore() {
    return setShowMore(function (prev) {
      return !prev;
    });
  };

  var handleChange = function handleChange(option) {
    return onChange(getValue(isMulti, option, value));
  };

  return /*#__PURE__*/React.createElement(Downshift, {
    id: props.id || props.name,
    onChange: handleChange,
    itemToString: function itemToString(value) {
      return _itemToString(value, isMulti, showMore, handleShowMore, handleChange);
    },
    selectedItem: value || '',
    stateReducer: function stateReducer(state, changes) {
      return _stateReducer(state, changes, isMulti);
    },
    onInputValueChange: function onInputValueChange(inputValue, _ref4) {
      var selectedItem = _ref4.selectedItem;

      /**
       * Prevent firing te load options callback when selecting value not filtering
       */
      if (onInputChange && typeof inputValue === 'string' && (selectedItem === null || selectedItem === void 0 ? void 0 : selectedItem.label) !== inputValue) {
        onInputChange(inputValue);
      }
    }
  }, function (_ref5) {
    var isOpen = _ref5.isOpen,
        inputValue = _ref5.inputValue,
        itemToString = _ref5.itemToString,
        selectedItem = _ref5.selectedItem,
        clearSelection = _ref5.clearSelection,
        getInputProps = _ref5.getInputProps,
        getToggleButtonProps = _ref5.getToggleButtonProps,
        getItemProps = _ref5.getItemProps,
        highlightedIndex = _ref5.highlightedIndex;
    var toggleButtonProps = getToggleButtonProps();
    return /*#__PURE__*/React.createElement("div", {
      className: "pf-v5-c-select"
    }, /*#__PURE__*/React.createElement("div", _extends({
      ref: selectToggleRef,
      disabled: isDisabled,
      className: "pf-v5-c-select__toggle".concat(isDisabled ? ' pf-v5-m-disabled' : '').concat(isSearchable ? ' pf-v5-m-typeahead' : '', " ddorg__pf4-component-mapper__select-toggle"),
      tabIndex: 0
    }, toggleButtonProps), /*#__PURE__*/React.createElement("div", {
      className: "pf-v5-c-select_toggle-wrapper ddorg__pf4-component-mapper__select-toggle-wrapper"
    }, /*#__PURE__*/React.createElement(ValueContainer, {
      isMulti: isMulti,
      isSearchable: isSearchable,
      placeholder: placeholder,
      inputRef: inputRef,
      getInputProps: getInputProps,
      value: itemToString(selectedItem, isMulti, showMore, handleShowMore, handleChange)
    })), isClearable && parsedValue && /*#__PURE__*/React.createElement(ClearIndicator, {
      clearSelection: clearSelection
    }), /*#__PURE__*/React.createElement("span", {
      className: "pf-v5-c-select__toggle-arrow"
    }, /*#__PURE__*/React.createElement(Icon, null, isFetching ? /*#__PURE__*/React.createElement(CircleNotchIcon, {
      className: "ddorg__pf4-component-mapper__select-loading-icon"
    }) : /*#__PURE__*/React.createElement(CaretDownIcon, null)))), isOpen && /*#__PURE__*/React.createElement(Menu, {
      noResultsMessage: noResultsMessage,
      noOptionsMessage: noOptionsMessage,
      isFetching: isFetching,
      isDisabled: isDisabled,
      isSearchable: isSearchable,
      getInputProps: getInputProps,
      filterOptions: filterOptions,
      filterValue: inputValue,
      options: options,
      getItemProps: getItemProps,
      highlightedIndex: highlightedIndex,
      selectedItem: isMulti ? value : parsedValue,
      isMulti: isMulti,
      menuPortalTarget: menuPortalTarget,
      menuIsPortal: menuIsPortal,
      selectToggleRef: selectToggleRef,
      originalOptions: originalOptions
    }));
  });
};

InternalSelect.propTypes = {
  onChange: PropTypes.func.isRequired,
  options: PropTypes.arrayOf(PropTypes.shape({
    value: PropTypes.any,
    label: PropTypes.any,
    divider: PropTypes.bool
  })).isRequired,
  value: PropTypes.any,
  simpleValue: PropTypes.bool,
  placeholder: PropTypes.string,
  isSearchable: PropTypes.bool,
  id: PropTypes.string,
  name: PropTypes.string.isRequired,
  isDisabled: PropTypes.bool,
  isClearable: PropTypes.bool,
  noResultsMessage: PropTypes.node,
  noOptionsMessage: PropTypes.func,
  isMulti: PropTypes.bool,
  isFetching: PropTypes.bool,
  onInputChange: PropTypes.func,
  loadingMessage: PropTypes.node,
  menuPortalTarget: PropTypes.any,
  menuIsPortal: PropTypes.bool,
  originalOptions: PropTypes.array
};

var Select = function Select(_ref6) {
  var _ref6$showMoreLabel = _ref6.showMoreLabel,
      showMoreLabel = _ref6$showMoreLabel === void 0 ? 'more' : _ref6$showMoreLabel,
      _ref6$showLessLabel = _ref6.showLessLabel,
      showLessLabel = _ref6$showLessLabel === void 0 ? 'Show less' : _ref6$showLessLabel,
      _ref6$simpleValue = _ref6.simpleValue,
      simpleValue = _ref6$simpleValue === void 0 ? true : _ref6$simpleValue,
      _ref6$loadingMessage = _ref6.loadingMessage,
      loadingMessage = _ref6$loadingMessage === void 0 ? 'Loading...' : _ref6$loadingMessage,
      _ref6$updatingMessage = _ref6.updatingMessage,
      updatingMessage = _ref6$updatingMessage === void 0 ? 'Loading data...' : _ref6$updatingMessage,
      _ref6$options = _ref6.options,
      options = _ref6$options === void 0 ? [] : _ref6$options,
      _ref6$menuIsPortal = _ref6.menuIsPortal,
      menuIsPortal = _ref6$menuIsPortal === void 0 ? false : _ref6$menuIsPortal,
      _ref6$placeholder = _ref6.placeholder,
      placeholder = _ref6$placeholder === void 0 ? 'Choose...' : _ref6$placeholder,
      _ref6$isSearchable = _ref6.isSearchable,
      isSearchable = _ref6$isSearchable === void 0 ? false : _ref6$isSearchable,
      _ref6$isClearable = _ref6.isClearable,
      isClearable = _ref6$isClearable === void 0 ? false : _ref6$isClearable,
      _ref6$noResultsMessag = _ref6.noResultsMessage,
      noResultsMessage = _ref6$noResultsMessag === void 0 ? 'No results found' : _ref6$noResultsMessag,
      _ref6$noOptionsMessag = _ref6.noOptionsMessage,
      noOptionsMessage = _ref6$noOptionsMessag === void 0 ? 'No options' : _ref6$noOptionsMessag,
      props = _objectWithoutProperties(_ref6, _excluded2);

  var menuPortalTarget = menuIsPortal ? document.body : undefined;
  return /*#__PURE__*/React.createElement(DataDrivenSelect, _extends({
    SelectComponent: InternalSelect,
    menuPortalTarget: menuPortalTarget,
    menuIsPortal: menuIsPortal
  }, props, {
    showMoreLabel: showMoreLabel,
    showLessLabel: showLessLabel,
    simpleValue: simpleValue,
    loadingMessage: loadingMessage,
    updatingMessage: updatingMessage,
    options: options,
    placeholder: placeholder,
    isSearchable: isSearchable,
    isClearable: isClearable,
    noResultsMessage: noResultsMessage,
    noOptionsMessage: noOptionsMessage,
    optionsTransformer: flatOptions
  }));
};

Select.propTypes = {
  isSearchable: PropTypes.bool,
  showMoreLabel: PropTypes.node,
  showLessLabel: PropTypes.node,
  simpleValue: PropTypes.bool,
  value: PropTypes.any,
  options: PropTypes.arrayOf(PropTypes.shape({
    value: PropTypes.any,
    label: PropTypes.any
  })),
  onChange: PropTypes.func.isRequired,
  isMulti: PropTypes.bool,
  loadOptions: PropTypes.func,
  loadingMessage: PropTypes.node,
  updatingMessage: PropTypes.node,
  menuIsPortal: PropTypes.bool,
  placeholder: PropTypes.string,
  noResultsMessage: PropTypes.node,
  noOptionsMessage: PropTypes.node,
  isClearable: PropTypes.bool
};
export default Select;